//
//! @file CCtrlHexEdit.h 
//! @copyright 1992 - 2011 Dennis Robinson (http://www.menasoft.com)
//

#ifndef _INC_CCtrlHexEdit_H
#define _INC_CCtrlHexEdit_H
#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "../GrayLibBase.h"
#ifdef _MFC_VER
#include "../../GrayCore/CHeap.h"
#include "../Geometry/CRectI.h"

// Resource.
#define IDR_POPUP_HEX_EDIT              500

class CCtrlHexEdit : public CEdit
{
public:
	CCtrlHexEdit();

	enum EDITMODE_TYPE
	{	
		EDIT_NONE,
		EDIT_ASCII,
		EDIT_HIGH,
		EDIT_LOW
	} ;
// Attributes
public:
	CHeapBlockLen m_Data;			//!< pointer to data
	int			m_topindex;			//!< offset of first visible byte on screen

	int			m_currentAddress;	//!< address under cursor
	EDITMODE_TYPE	m_currentMode;		//!< current editing mode: address/hex/ascii
	int			m_selStart;			//!< start address of selection
	int			m_selEnd;			//!< end address of selection

	size_t		m_BytesPerRow;				//!< byte per row 
	int			m_LinesPerPage;				//!< lines per page
	BOOL		m_bShowAddress;
	BOOL		m_bShowAscii;
	BOOL		m_bShowHex;
	BOOL		m_bAddressIsWide;
	
	BOOL		m_bNoAddressChange;	//!< internally used
	BOOL		m_bHalfPage;
	
	CFont		m_Font;
	int			m_lineHeight;
	int			m_nullWidth;
	BOOL		m_bUpdate;

	int			m_offHex,
				m_offAscii,
				m_offAddress;

	CPointI		m_editPos;

// Operations
public:
	BYTE* GetDataPtr() const;

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CCtrlHexEdit)
	protected:
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	//}}AFX_VIRTUAL

// Implementation
public:
	int GetData(LPBYTE p, int len);
	void SetData(LPBYTE p, int len);
	CSize GetSel(void);
	void SetSel(int s, int e);
	void SetBPR( size_t bpr );
	void SetOptions(BOOL a, BOOL h, BOOL c, BOOL w);
	virtual ~CCtrlHexEdit();

	// Generated message map functions
protected:
	void		ScrollIntoView(int p);
	void		RepositionCaret(int p);
	void		Move(int x, int y);
	inline BOOL IsSelected(void);
	void		UpdateScrollBars(void);
	void		CreateEditCaret(void);
	void		CreateAddressCaret(void);
	CPointI		CalcPos(int x, int y);
	void		SelInsert(int s, int l);
	void		SelDelete(int s, int e);
	inline void NormalizeSel(void);

	//{{AFX_MSG(CCtrlHexEdit)
	afx_msg void OnContextMenu(CWnd*, CPoint point);
	afx_msg void OnChar(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnKillFocus(CWnd* pNewWnd);
	afx_msg void OnPaint();
	afx_msg void OnSetFocus(CWnd* pOldWnd);
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg UINT OnGetDlgCode();
	afx_msg BOOL OnEraseBkgnd(CDC* pDC);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnEditClear();
	afx_msg void OnEditCopy();
	afx_msg void OnEditCut();
	afx_msg void OnEditPaste();
	afx_msg void OnEditSelectAll();
	afx_msg void OnEditUndo();
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()
};

#endif	
#endif	// _INC_CCtrlHexEdit_H

