//
//! @file CHyperLink.h
//! HyperLink static control. Will open the default browser with the given URL
//! when the user clicks on the link.
//! @copyright 1992 - 2011 Dennis Robinson (http://www.menasoft.com)
//

#ifndef _INC_CHyperLink_H
#define _INC_CHyperLink_H
#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "../GrayLibBase.h"
#ifdef _MFC_VER
#include "../Geometry/CRectI.h"
#include "../../GrayCore/CString.h"

class CHyperLink : public CStatic
{
	//! @class Gray::CHyperLink
	//! MFC CHyperLink window

	typedef CStatic SUPER_t;

public:
    CHyperLink();
    virtual ~CHyperLink();

public:
    enum UnderLine_TYPE
	{
		UL_Hover = -1, UL_None = 0, UL_Always = 1
	};

public:
    void SetURL(CGString strURL);
    CGString GetURL() const
	{ 
		return m_strURL;   
	}

    void SetColours( COLORREF crLinkColour, COLORREF crVisitedColour, COLORREF crHoverColour = -1 );
    COLORREF GetLinkColour() const
	{ 
	    return m_crLinkColour; 
	}
    COLORREF GetVisitedColour() const
	{
	    return m_crVisitedColour; 
	}
    COLORREF GetHoverColour() const
	{
		return m_crHoverColour;
	}

    void SetVisited(BOOL bVisited = true);
    BOOL GetVisited() const
	{ 
		return m_bVisited; 
	}

    void SetLinkCursor(HCURSOR hCursor);
    HCURSOR GetLinkCursor() const
	{
	    return m_hLinkCursor;
	}

    void SetUnderline( UnderLine_TYPE nUnderline = UL_Hover );
    UnderLine_TYPE GetUnderline() const
	{ 
		return m_nUnderline; 
	}

    void SetAutoSize(BOOL bAutoSize = true);
    BOOL GetAutoSize() const
	{ 
		return m_bAdjustToFit; 
	}

// Overrides
    // ClassWizard generated virtual function overrides
    //{{AFX_VIRTUAL(CHyperLink)
	public:
    virtual BOOL PreTranslateMessage(MSG* pMsg);
	virtual BOOL DestroyWindow();
	protected:
    virtual void PreSubclassWindow();
	//}}AFX_VIRTUAL

// Implementation
protected:
    void ReportError(HRESULT nError);
    void PositionWindow();
    void SetDefaultCursor();

// Protected attributes
protected:
	// exposed params.
    CGString  m_strURL;                     //!< hyperlink URL. doesn't have to match text.
    COLORREF m_crLinkColour;
	COLORREF m_crVisitedColour;				//!< Hyperlink colours
    COLORREF m_crHoverColour;               //!< Hover colour
    BOOL     m_bVisited;                    //!< Has it been visited?
    UnderLine_TYPE m_nUnderline;          //!< underline hyperlink?
    HCURSOR  m_hLinkCursor;                 //!< Cursor for hyperlink
    BOOL     m_bAdjustToFit;                //!< Adjust window size to fit text?

	// internal vars.
    CFont    m_UnderlineFont;               //!< Font for underline display
    CFont    m_StdFont;                     //!< Standard font
    CToolTipCtrl m_ToolTip;                 //!< The tooltip
    UINT     m_nTimerID;
    BOOL     m_bOverControl;                //!< cursor over control?

    // Generated message map functions
protected:
    //{{AFX_MSG(CHyperLink)
    afx_msg HBRUSH CtlColor(CDC* pDC, UINT nCtlColor);
    afx_msg BOOL OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message);
    afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnTimer(UINT_PTR nIDEvent);
	afx_msg BOOL OnEraseBkgnd(CDC* pDC);
	//}}AFX_MSG
    afx_msg void OnClicked();
    DECLARE_MESSAGE_MAP()
};

//*******************************************
//{{AFX_INSERT_LOCATION}}
// Microsoft Developer Studio will insert additional declarations immediately before the previous line.

#endif // _INC_CHyperLink_H
#endif
